/* CIUPERCA Cristian-Adrian 334CB */

#include <stdio.h>
#include <stdlib.h>
#include "ili9341.h"
#include "ili9341gfx.h"
#include "usart.h"
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>


#define SPEED ((100 - distance)/10)
#define MINROT 200
#define MAXROT 650
#define RATIO 2.505f


volatile unsigned long int delta = 0;
volatile unsigned long int useconds = 0;
volatile int centimeters = 0;
volatile int check = 0, seconds = 0;
volatile int clockwise = 0;
volatile int rotation_pos = 200;
volatile float degree = 200.0f;
volatile float angle = 0;

volatile int d1 = 0, d2 = 0, d3 = 0;


void Wait(int time)
{
	_delay_ms (time);	
}

void lights_on()
{
	float rat = 100/42;
	if(d1 != 0) {
		if (d1 < 20 * rat)
			PORTA |= (1 << PA3);
		else if (d1 < 50 * rat)
			PORTA |= (1 << PA4);
		else 
			PORTA |= (1 << PA5);
	}
		
	if(d2 != 0) {
		if (d2 < 20 * rat)
			PORTA |= (1 << PA0);
		else if (d2 < 50 * rat)
			PORTA |= (1 << PA1);
		else 
			PORTA |= (1 << PA2);
	}
	
	if(d3 != 0) {
		if (d3 < 20 * rat)
			PORTB |= (1 << PB4);
		else if (d3 < 50 * rat)
			PORTB |= (1 << PB5);
		else 
			PORTB |= (1 << PB6);
	}
	
	PORTB |= (1 << PB7);
}

void lights_off()
{
	PORTB  &= ~(1 << PB4);
	PORTB  &= ~(1 << PB5);
	PORTB  &= ~(1 << PB6);
	PORTB  &= ~(1 << PB7);
	
	PORTA  &= ~(1 << PA0);
	PORTA  &= ~(1 << PA1);
	PORTA  &= ~(1 << PA2);
	PORTA  &= ~(1 << PA3);
	PORTA  &= ~(1 << PA4);
	PORTA  &= ~(1 << PA5);
}

int spin(int wait)
{
	int spd = 1;
	//if (SPEED > 0)
	//	spd = SPEED;
	if (clockwise == 0) {
		degree += RATIO * spd;
		angle += 1 * spd;
		rotation_pos = (int)degree;
		if (rotation_pos == MAXROT)
			clockwise = 1;
	} else {
		
		degree -= RATIO * spd;
		angle -= 1 * spd;
		rotation_pos = (int)degree;
		if (rotation_pos == MINROT)
			clockwise = 0;
	}
	Wait(wait);
	
	return rotation_pos;
}


void servo_init()
{
	// prescaler 64 fast pwm 
	TCCR1A|=(1<<COM1A1)|(1<<COM1B1)|(1<<WGM11);
	TCCR1B|=(1<<WGM13)|(1<<WGM12)|(1<<CS11)|(1<<CS10);
	
	// freq = 50 Hz
	ICR1=4999;
	
	DDRD|=(1<<PD4)|(1<<PD5);

}

void servo_reset_pos()
{
	OCR1A = MINROT;
	Wait(5000);
}


ISR(TIMER2_COMPA_vect)
{
	useconds ++;
}

void init_timer2(void)
{	
	OCR2A = 0x10;
	TCCR2A = (1 << WGM21);
	TCCR2B = (1 << CS21) | (1 << CS20);
	TIMSK2 = (1 << OCIE2A); 
}


ISR(INT2_vect) {


	if((PINB & (1 << PB2)) == 0 ) {
		check = 2;
		delta = useconds;
		centimeters = (int)((float) delta / 1.8f);
		
	} else {
		useconds = 0;
		check = 1;
	}
}


void init_sensors() {

	EIMSK |= (1 << INT2);
	
	//intrerupere
	EICRA |= (1 << ISC20);

	//trigger pins
	DDRB |= (1 << PB1);
	DDRB |= (1 << PB3);
	
	//pin echo
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);

}


int get_distance() {

	check = 0;
	centimeters = 0;
	
	lights_on();
	
	PORTB &= ~(1 << PB1);
	_delay_us(2);
	
	PORTB |= (1 << PB1);
	_delay_us(10);
	PORTB &= ~(1 << PB1);

	while(check != 2);
	if (angle == 0)
		d1 = delta;
	if (angle == 90)
		d2 = delta;
	if (angle == 180)
		d3 = delta;
		
	lights_off();
	
	return delta;
}

int get_distance2() {

	check = 0;
	delta = 0;
	
	PORTB &= ~(1 << PB3);
	_delay_us(2);
	
	PORTB |= (1 << PB3);
	_delay_us(10);
	PORTB &= ~(1 << PB3);

	while(check != 2);
	
	return delta;
}

int get_accurate_distance() {
	int d1, d2, d3, daux;
	int done = 0;
	while (done != 3) {
		daux = get_distance();
		//if (centimeters > 400) {
		//	return 400;
		//} else {
			switch(done){
				case 0:
					d1 = daux;
					done++;
					break;
				case 1:
					d2 = daux;
					done++;
					break;
				case 2:
					d3 = daux;
					return (d1 + d2 + d3) / 3;
			}
		//}
	}
	return 0;
}


void init_buzzer()
{
	DDRB |= (1 << PB0);
}

void buzz(int x)
{
//	int i;
//	for (i = 0; i < 50; i++) {
		PORTB |= (1 << PB0);
		_delay_us(x);
		PORTB  &= ~(1 << PB0);
		_delay_us(x);
//	}
}

void init_leds()
{
	DDRB |= (1 << PB4);
	DDRB |= (1 << PB5);
	DDRB |= (1 << PB6);
	DDRB |= (1 << PB7);
	
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	DDRA |= (1 << PA3);
	DDRA |= (1 << PA4);
	DDRA |= (1 << PA5);

}

void lightup()
{
		PORTB |= (1 << PB4);
		PORTB |= (1 << PB5);
		PORTB |= (1 << PB6);
		
		PORTA |= (1 << PA0);
		PORTA |= (1 << PA1);
		PORTA |= (1 << PA2);
		PORTA |= (1 << PA3);
		PORTA |= (1 << PA4);
		PORTA |= (1 << PA5);
		
		_delay_us(10);
		
		PORTB  &= ~(1 << PB4);
		PORTB  &= ~(1 << PB5);
		PORTB  &= ~(1 << PB6);
		
		PORTA  &= ~(1 << PA0);
		PORTA  &= ~(1 << PA1);
		PORTA  &= ~(1 << PA2);
		PORTA  &= ~(1 << PA3);
		PORTA  &= ~(1 << PA4);
		PORTA  &= ~(1 << PA5);
		
		_delay_us(10);
}


int main(void)
{
	
	USART0_init();
	servo_init();
	init_timer2();
	init_sensors();
	init_leds();
	sei();
	
	init_buzzer();
	int dist = 0, dist2 = 0;
	char buffer[33];
	float rat = 100/42;
	
	servo_reset_pos();
	//PORTB |= (1 << PB5);
				
				
	//PORTB |= (1 << PB4);
	while(1){
		
		dist = get_distance();
		OCR1A = spin(0);
		buzz(dist/10);

		
	}

	
	return 0;
}
